<?php

class Paygine
{
	static $pluginName = 'payment-paygine';
	const TEST_URL = 'https://test.paygine.com';
	const PROD_URL = 'https://pay.paygine.com';
	// Стандартные статусы заказов
	const NOT_CONFIRMED = 0;		// не подтвержден
	const EXPECTS_PAYMENT = 1;	// ожидает оплаты
	const PAID = 2;							// оплачен
	const IN_DELIVERY = 3;			//в доставке
	const CANCELED = 4;					//отменен
	const EXECUTED = 5;					//выполнен
	const PROCESSING = 6;				//в обработке
	
	const ORDER_REGISTERED = 'REGISTERED';
	const ORDER_AUTHORIZED = 'AUTHORIZED';
	const ORDER_COMPLETED = 'COMPLETED';
	const ORDER_REFUNDED = 'CANCELED';
	const OPERATION_APPROVED = 'APPROVED';
	const OPERATION_COMPLETE = 'COMPLETE';
	const OPERATION_REFUND = 'REVERSE';
	
	/**
	 * @param $payment_param
	 * @param $order
	 * @param $orderContent
	 * @return bool|string
	 */
	public static function registerOrder($payment_param, $order, $orderContent)
	{
		$amount = self::centify($order['summ']) + self::centify($order['delivery_cost']);
		$currency = self::getCurrency($order['currency_iso']);
		$signature = self::signHash($payment_param['sector'] . $amount . $currency . $payment_param['password'], $payment_param['hashing']);
		$fiscal_positions = self::calcFiscalPositions($payment_param, $order, $orderContent);
		$email = !empty($order['contact_email']) ? $order['contact_email'] : (!empty($order['user_email']) ? $order['user_email'] : '');
		
		$data = [
			'sector' => $payment_param['sector'],
			'reference' => $order['id'],
			'fiscal_positions' => $fiscal_positions,
			'amount' => $amount,
			'description' => $order['number'],
			'email' => $email,
			'phone' => !empty($order['phone']) ? $order['phone'] : '',
			'currency' => $currency,
			'mode' => 1,
			'url' => SITE . '/payment?payment=' . self::$pluginName,
			'signature' => $signature,
		];
		
		return self::sendRequest(self::getPaymentUrl($payment_param['test_mode']) . '/webapi/Register', $data);
	}
	
	/**
	 * @param $value
	 * @return int
	 */
	public static function centify($value) {
		return intval(round($value, 2) * 100);
	}
	
	/**
	 * @param string $iso
	 * @return string
	 */
	public static function getCurrency($iso)
	{
		switch(strtoupper($iso)) {
			case 'RUR':
			case 'RUB': return '643';
			case 'EUR': return '978';
			case 'USD': return '840';
			
		}
		return '643';
	}
	
	/**
	 * @param $data
	 * @param $hashing
	 * @return string
	 */
	private static function signHash($data, $hashing = 'md5') {
		return base64_encode(hash($hashing, $data));
	}
	
	/**
	 * @param array $data
	 * @param string $password
	 * @return void
	 */
	public static function signData(&$data, $hashing = 'md5', $password = '') {
		$sign = implode('', $data);
		if($password) $sign .= $password;
		$data['signature'] = self::signHash($sign, $hashing);
	}
	
	/**
	 * @param array|string $data
	 * @param string $password
	 * @return string
	 */
	private static function generateSign($data, $hashing = 'md5', $password = '') {
		$string = is_array($data) ? implode('', $data) : (string) $data;
		if($password) $string .= $password;
		return base64_encode(hash($hashing, $string));
	}
	
	/**
	 * @param $payment_param
	 * @param $order
	 * @param $orderContent
	 * @return false|string
	 */
	private static function calcFiscalPositions($payment_param, $order, $orderContent) {
		$tax = $payment_param['tax'];
		$order_amount = self::centify($order['summ']);
		$fiscal_positions = '';
		$fiscal_amount = 0;
		foreach($orderContent as $product) {
			$fiscal_positions .= $product['count'] . ';';
			$element_price = self::centify($product['price']);
			$fiscal_positions .= $element_price . ';';
			$fiscal_positions .= $tax . ';';
			$fiscal_positions .= str_replace([';', '|'], '', $product['name']) . '|';
			$fiscal_amount += $product['count'] * $element_price;
		}
		
		if(!empty($order['delivery_cost'])){
			$fiscal_positions .= '1;';
			$element_price = self::centify($order['delivery_cost']);
			$fiscal_positions .= $element_price . ';';
			$fiscal_positions .= $tax . ';';
			$fiscal_positions .= 'Доставка' . '|';
			$fiscal_amount += $element_price;
			$order_amount += $element_price;
		}
		$fiscal_diff = abs($fiscal_amount - $order_amount);
		if ($fiscal_diff)
			$fiscal_positions .= '1;' . $fiscal_diff . ';6;' . 'Скидка' . ';14|';
		return substr($fiscal_positions, 0, -1);
	}
	
	/**
	 * @param $order
	 * @param $orderContent
	 * @return array
	 */
	public static function calcShopCart($order, $orderContent) {
		$order_amount = self::centify($order['summ']);
		$shop_cart = [];
		$fiscal_amount = 0;
		$sc_key = 0;
		foreach($orderContent as $product) {
			$element_price = self::centify($product['price']);
			$fiscal_amount += $product['count'] * $element_price;
			
			$shop_cart[$sc_key]['name'] = $product['name'];
			$shop_cart[$sc_key]['quantityGoods'] = (int) $product['count'];
			$shop_cart[$sc_key]['goodCost'] = round($product['price'] * $shop_cart[$sc_key]['quantityGoods'], 2);
			$sc_key++;
		}
		
		if(!empty($order['delivery_cost'])){
			$element_price = self::centify($order['delivery_cost']);
			$fiscal_amount += $element_price;
			
			$shop_cart[$sc_key]['quantityGoods'] = 1;
			$shop_cart[$sc_key]['goodCost'] = $order['delivery_cost'];
			$shop_cart[$sc_key]['name'] = 'Доставка';
		}
		$fiscal_diff = abs($fiscal_amount - $order_amount);
		if ($fiscal_diff) return [];
		return $shop_cart;
	}
	
	/**
	 * @param $url
	 * @param $data
	 * @return bool|string
	 */
	public static function sendRequest($url, $data) {
		$response = false;
		$repeat = 3;
		while ($repeat) {
			$repeat--;
			
			if( $curl = curl_init() ) {
				curl_init();
				curl_setopt($curl, CURLOPT_URL, $url);
				curl_setopt($curl, CURLOPT_RETURNTRANSFER,true);
				curl_setopt($curl, CURLOPT_POST, true);
				curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));
				$response = curl_exec($curl);
				curl_close($curl);
			}
			
			if ($response) break;
			sleep(2);
		}
		return $response;
	}
	
	/**
	 * @param $test_mode
	 * @return string
	 */
	public static function getPaymentUrl($test_mode) {
		return $test_mode ? self::TEST_URL : self::PROD_URL;
	}
	
	/**
	 * @param string $payment_method
	 * @return string
	 */
	public static function getPaymentPath($payment_method = 'acquiring') {
		switch($payment_method){
			case 'acquiring': return '/webapi/Purchase';
			case 'acquiring_two_stage': return '/webapi/Authorize';
			case 'svkb': return '/webapi/custom/svkb/PurchaseWithInstallment';
			case 'svkb_two_stage': return '/webapi/custom/svkb/AuthorizeWithInstallment';
			case 'sbp': return '/webapi/PurchaseSBP';
		}
	}
	
	/**
	 * @param $order_state
	 * @param $payment_param
	 * @return false|mixed
	 */
	public static function getCustomOrderStatus($order_state, $payment_param) {
		switch($order_state){
			case self::ORDER_AUTHORIZED: return $payment_param['order_authorized'];
			case self::ORDER_COMPLETED: return $payment_param['order_completed'];
			case self::ORDER_REFUNDED: return $payment_param['order_canceled'];
		}
		return false;
	}
	
	/**
	 * @param $string
	 * @return mixed
	 * @throws Exception
	 */
	public static function parseXML($string) {
		if (!$string)
			throw new Exception('Пустая строка');
		$xml = simplexml_load_string($string);
		if (!$xml)
			throw new Exception('Не удалось получить данные из XML');
		$valid_xml = json_decode(json_encode($xml), true);
		if (!$valid_xml)
			throw new Exception('Данные в XML не валидны');
		return $valid_xml;
	}
	
	/**
	 * @param $array
	 * @return string
	 */
	private static function xmlArrayToString($array) {
		$res = '';
		foreach($array as $key => $value) {
			if($key === '@attributes' or $key === 'signature') continue;
			$res .= is_string($value) ? $value : self::xmlArrayToString($value);
		}
		return $res;
	}
	
	/**
	 * @param array $response
	 * @return void
	 * @throws Exception
	 */
	public static function responseValidation($response, $payment_param, $is_operation = true) {
		if($is_operation){
			if(empty($response['reason_code']) && !empty($response['code']) && !empty($response['description']))
				throw new Exception("$response[description] (error code $response[code])");
		//	if(!in_array($response['type'], self::OPERATIONS_TYPES))
		//		throw new Exception('Неизвестный тип операции : ' . $response['type']);
		}
		if(empty($response['signature']))
			throw new Exception('Пустая подпись');
		$xml_string = self::xmlArrayToString($response);
		$signature = self::generateSign($xml_string, $payment_param['hashing'], $payment_param['password']);
		if ($signature !== $response['signature'])
			throw new Exception('Подпись не валидна');
	}
	
	/**
	 * @param $pc_order_id
	 * @param $payment_param
	 * @return mixed
	 * @throws Exception
	 */
	public static function getOrderFromPc($pc_order_id, $payment_param) {
		if(!$pc_order_id)
			throw new Exception('PC order ID undefined');
		
		$data = [
			'sector' => $payment_param['sector'],
			'id' => $pc_order_id
		];
		self::signData($data, $payment_param['hashing'], $payment_param['password']);
		$response = self::sendRequest(self::getPaymentUrl($payment_param['test_mode']) . '/webapi/Order', $data);
		return self::parseXML($response);
	}
	
	/**
	 * @param $order_id
	 * @param $payment_id
	 * @return false|mixed
	 */
	public static function getOrder($order_id, $payment_id) {
		$orderModel = new Models_Order();
		$orders = $orderModel->getOrder(
			'`id` = ' . DB::quoteInt($order_id) . ' AND ' .
			'`payment_id` = ' . DB::quoteInt($payment_id)
		);
		return (!$orders || empty($orders[$order_id])) ? false : $orders[$order_id];
	}
	
	/**
	 * @return array[]
	 */
	public static function getOrderStatusList() {
		return [
			[
				'title' => lang('paymentWaiting'),
				'value' => self::EXPECTS_PAYMENT,
			],
			[
				'title' => lang('paymentPaid'),
				'value' => self::PAID,
			],
			[
				'title' => lang('paymentDelivering'),
				'value' => self::IN_DELIVERY,
			],
			[
				'title' => lang('paymentCanceled'),
				'value' => self::CANCELED,
			],
			[
				'title' => lang('paymentCompleted'),
				'value' => self::EXECUTED,
			],
			[
				'title' => lang('paymentProcessing'),
				'value' => self::PROCESSING,
			]
		];
	}
	
	/**
	 * @param $url
	 * @return void
	 */
	static function jsRedirect($url){
		exit("<script>window.top.location.href = '{$url}'</script>");
	}
	
	/**
	 * @param $action
	 * @param $content
	 * @return void
	 */
	public static function log($action, $content){
		$paymentLogsDir = SITE_DIR.TEMP_DIR.'payLogs';
		if (!file_exists($paymentLogsDir))
			@mkdir($paymentLogsDir, 0755);
		$paymentLogsFile = $paymentLogsDir.DS.self::$pluginName.'.log';
		$logData = '###LOG '.date('Y-m-d H:i:s')."\n";
		$logData .= '###ACTION '.$action."\n";
		$logData .= '###CONTENT_START'."\n";
		$logData .= print_r($content, true)."\n";
		$logData .= '###CONTENT_END'."\n\n";
		file_put_contents($paymentLogsFile, $logData, FILE_APPEND);
	}
}